@@ -57,7 +57,7 @@
 #define TRACKER_DB_PAGE_SIZE_DONT_SET -1
 
 /* Set current database version we are working with */
-#define TRACKER_DB_VERSION_NOW        TRACKER_DB_VERSION_0_9_34
+#define TRACKER_DB_VERSION_NOW        TRACKER_DB_VERSION_0_9_38
 #define TRACKER_DB_VERSION_FILE       "db-version.txt"
 #define TRACKER_DB_LOCALE_FILE        "db-locale.txt"
 
@@ -97,7 +97,8 @@ typedef enum {
 	TRACKER_DB_VERSION_0_9_19,  /* collation */
 	TRACKER_DB_VERSION_0_9_21,  /* Fix for NB#186055 */
 	TRACKER_DB_VERSION_0_9_24,  /* nmo:PhoneMessage class */
-	TRACKER_DB_VERSION_0_9_34   /* ontology cache */
+	TRACKER_DB_VERSION_0_9_34,  /* ontology cache */
+	TRACKER_DB_VERSION_0_9_38   /* nie:url an inverse functional property */
 } TrackerDBVersion;
 
 typedef struct {
@@ -139,13 +140,16 @@ static TrackerDBDefinition dbs[] = {
 	  0 },
 };
 
-static gboolean            db_exec_no_reply    (TrackerDBInterface *iface,
-                                                const gchar        *query,
-                                                ...);
-static TrackerDBInterface *db_interface_create (TrackerDB           db);
-static TrackerDBInterface *tracker_db_manager_get_db_interfaces     (gint num, ...);
-static TrackerDBInterface *tracker_db_manager_get_db_interfaces_ro  (gint num, ...);
-static void                db_remove_locale_file  (void);
+static gboolean            db_exec_no_reply                        (TrackerDBInterface   *iface,
+                                                                    const gchar          *query,
+                                                                    ...);
+static TrackerDBInterface *db_interface_create                      (TrackerDB            db,
+                                                                     GError             **error);
+static TrackerDBInterface *tracker_db_manager_get_db_interfaces     (GError             **error,
+                                                                     gint                 num, ...);
+static TrackerDBInterface *tracker_db_manager_get_db_interfaces_ro  (GError             **error,
+                                                                     gint                 num, ...);
+static void                db_remove_locale_file                    (void);
 
 static gboolean              initialized;
 static gboolean              locations_initialized;
@@ -207,9 +211,10 @@ tracker_db_manager_get_flags (guint *select_cache_size, guint *update_cache_size
 }
 
 static void
-db_set_params (TrackerDBInterface *iface,
-               gint                cache_size,
-               gint                page_size)
+db_set_params (TrackerDBInterface   *iface,
+               gint                  cache_size,
+               gint                  page_size,
+               GError              **error)
 {
 	gchar *queries = NULL;
 	const gchar *pragmas_file;
@@ -226,7 +231,8 @@ db_set_params (TrackerDBInterface *iface,
 		}
 		g_free (queries);
 	} else {
-		GError *error = NULL;
+		GError *internal_error = NULL;
+		TrackerDBStatement *stmt;
 
 		tracker_db_interface_execute_query (iface, NULL, "PRAGMA synchronous = OFF;");
 		tracker_db_interface_execute_query (iface, NULL, "PRAGMA count_changes = 0;");
@@ -234,12 +240,31 @@ db_set_params (TrackerDBInterface *iface,
 		tracker_db_interface_execute_query (iface, NULL, "PRAGMA encoding = \"UTF-8\"");
 		tracker_db_interface_execute_query (iface, NULL, "PRAGMA auto_vacuum = 0;");
 
-		tracker_db_interface_execute_query (iface, &error, "PRAGMA journal_mode = WAL;");
-		if (error) {
-			/* Don't just silence the problem. This pragma must return 'WAL' */
+		stmt = tracker_db_interface_create_statement (iface, TRACKER_DB_STATEMENT_CACHE_TYPE_NONE,
+		                                              &internal_error,
+		                                              "PRAGMA journal_mode = WAL;");
+
+		if (internal_error) {
 			g_message ("Can't set journal mode to WAL: '%s'",
-			           error->message);
-			g_clear_error (&error);
+			           internal_error->message);
+			g_propagate_error (error, internal_error);
+		} else {
+			TrackerDBCursor *cursor;
+
+			cursor = tracker_db_statement_start_cursor (stmt, NULL);
+			if (tracker_db_cursor_iter_next (cursor, NULL, NULL)) {
+				if (g_ascii_strcasecmp (tracker_db_cursor_get_string (cursor, 0, NULL), "WAL") != 0) {
+					g_set_error (error,
+					             TRACKER_DB_INTERFACE_ERROR,
+					             TRACKER_DB_OPEN_ERROR,
+					             "Can't set journal mode to WAL");
+				}
+			}
+			g_object_unref (cursor);
+		}
+
+		if (stmt) {
+			g_object_unref (stmt);
 		}
 
 		if (page_size != TRACKER_DB_PAGE_SIZE_DONT_SET) {
@@ -272,11 +297,13 @@ db_type_to_string (TrackerDB db)
 }
 
 static TrackerDBInterface *
-db_interface_get (TrackerDB  type,
-                  gboolean  *create)
+db_interface_get (TrackerDB   type,
+                  gboolean   *create,
+                  GError    **error)
 {
 	TrackerDBInterface *iface;
-	const gchar        *path;
+	const gchar *path;
+	GError *internal_error = NULL;
 
 	path = dbs[type].abs_filename;
 
@@ -291,35 +318,62 @@ db_interface_get (TrackerDB  type,
 	           path,
 	           db_type_to_string (type));
 
-	iface = tracker_db_interface_sqlite_new (path);
+	iface = tracker_db_interface_sqlite_new (path,
+	                                         &internal_error);
+
+	if (internal_error) {
+		g_propagate_error (error, internal_error);
+		return NULL;
+	}
 
 	db_set_params (iface,
 	               dbs[type].cache_size,
-	               dbs[type].page_size);
+	               dbs[type].page_size,
+	               &internal_error);
+
+	if (internal_error) {
+		g_propagate_error (error, internal_error);
+		return NULL;
+	}
 
 	return iface;
 }
 
 static TrackerDBInterface *
-db_interface_get_metadata (void)
+db_interface_get_metadata (GError **error)
 {
 	TrackerDBInterface *iface;
-	gboolean            create;
+	gboolean create;
+	GError *internal_error = NULL;
 
-	iface = db_interface_get (TRACKER_DB_METADATA, &create);
+	iface = db_interface_get (TRACKER_DB_METADATA, &create, &internal_error);
+
+	if (internal_error) {
+		g_propagate_error (error, internal_error);
+		return NULL;
+	}
 
 	return iface;
 }
 
 static TrackerDBInterface *
-db_interface_create (TrackerDB db)
+db_interface_create (TrackerDB db,
+                     GError **error)
 {
+	TrackerDBInterface *iface;
+	GError *internal_error = NULL;
+
 	switch (db) {
 	case TRACKER_DB_UNKNOWN:
 		return NULL;
 
 	case TRACKER_DB_METADATA:
-		return db_interface_get_metadata ();
+		iface = db_interface_get_metadata (&internal_error);
+		if (internal_error) {
+			g_propagate_error (error, internal_error);
+			return NULL;
+		}
+		return iface;
 
 	default:
 		g_critical ("This TrackerDB type:%d->'%s' has no interface set up yet!!",
@@ -338,6 +392,7 @@ db_manager_remove_journal (void)
 	gboolean do_rotate = FALSE;
 	const gchar *dirs[3] = { NULL, NULL, NULL };
 	guint i;
+	GError *error = NULL;
 
 	/* We duplicate the path here because later we shutdown the
 	 * journal which frees this data. We want to survive that.
@@ -352,7 +407,14 @@ db_manager_remove_journal (void)
 	directory = g_path_get_dirname (path);
 
 	tracker_db_journal_get_rotating (&do_rotate, &chunk_size, &rotate_to);
-	tracker_db_journal_shutdown ();
+	tracker_db_journal_shutdown (&error);
+
+	if (error) {
+		/* TODO: propagate error */
+		g_message ("Ignored error while shutting down journal during remove: %s",
+		           error->message ? error->message : "No error given");
+		g_error_free (error);
+	}
 
 	dirs[0] = directory;
 	dirs[1] = do_rotate ? rotate_to : NULL;
@@ -654,10 +716,11 @@ tracker_db_get_type (void)
 }
 
 static void
-db_recreate_all (void)
+db_recreate_all (GError **error)
 {
 	guint i;
 	gchar *locale;
+	GError *internal_error = NULL;
 
 	/* We call an internal version of this function here
 	 * because at the time 'initialized' = FALSE and that
@@ -671,7 +734,19 @@ db_recreate_all (void)
 	g_message ("Creating database files, this may take a few moments...");
 
 	for (i = 1; i < G_N_ELEMENTS (dbs); i++) {
-		dbs[i].iface = db_interface_create (i);
+		dbs[i].iface = db_interface_create (i, &internal_error);
+		if (internal_error) {
+			guint y;
+
+			for (y = 1; y < i; y++) {
+				g_object_unref (dbs[y].iface);
+				dbs[y].iface = NULL;
+			}
+
+			g_propagate_error (error, internal_error);
+
+			return;
+		}
 	}
 
 	/* We don't close the dbs in the same loop as before
@@ -717,26 +792,60 @@ tracker_db_manager_init_locations (void)
 	locations_initialized = TRUE;
 }
 
-gboolean
-tracker_db_manager_init (TrackerDBManagerFlags  flags,
-                         gboolean              *first_time,
-                         gboolean               shared_cache,
-                         guint                  select_cache_size,
-                         guint                  update_cache_size,
-                         TrackerBusyCallback    busy_callback,
-                         gpointer               busy_user_data,
-                         const gchar           *busy_operation)
+static void
+perform_recreate (gboolean *first_time, GError **error)
 {
-	GType               etype;
-	TrackerDBVersion    version;
-	gchar              *filename;
-	const gchar        *dir;
-	const gchar        *env_path;
-	gboolean            need_reindex;
-	guint               i;
-	int                 in_use_file;
-	gboolean            loaded = FALSE;
+	GError *internal_error = NULL;
+	guint i;
+
+	if (first_time) {
+		*first_time = TRUE;
+	}
+
+	for (i = 1; i < G_N_ELEMENTS (dbs); i++) {
+		if (dbs[i].iface) {
+			g_object_unref (dbs[i].iface);
+			dbs[i].iface = NULL;
+		}
+	}
+
+	if (!tracker_file_system_has_enough_space (data_dir, TRACKER_DB_MIN_REQUIRED_SPACE, TRUE)) {
+		g_set_error (error,
+		             TRACKER_DB_INTERFACE_ERROR,
+		             TRACKER_DB_OPEN_ERROR,
+		             "Filesystem has not enough space");
+		return;
+	}
+
+	db_recreate_all (&internal_error);
+
+	if (internal_error) {
+		g_propagate_error (error, internal_error);
+	}
+}
+
+gboolean
+tracker_db_manager_init (TrackerDBManagerFlags   flags,
+                         gboolean               *first_time,
+                         gboolean                shared_cache,
+                         guint                   select_cache_size,
+                         guint                   update_cache_size,
+                         TrackerBusyCallback     busy_callback,
+                         gpointer                busy_user_data,
+                         const gchar            *busy_operation,
+                         GError                **error)
+{
+	GType etype;
+	TrackerDBVersion version;
+	gchar *filename;
+	const gchar *dir;
+	const gchar *env_path;
+	gboolean need_reindex;
+	guint i;
+	int in_use_file;
+	gboolean loaded = FALSE;
 	TrackerDBInterface *resources_iface;
+	GError *internal_error = NULL;
 
 	/* First set defaults for return values */
 	if (first_time) {
@@ -792,39 +901,44 @@ tracker_db_manager_init (TrackerDBManagerFlags  flags,
 	                             "tracker",
 	                             NULL);
 
+	g_free (in_use_filename);
 	in_use_filename = g_build_filename (g_get_user_data_dir (),
 	                                    "tracker",
 	                                    "data",
 	                                    IN_USE_FILENAME,
 	                                    NULL);
 
-	/* Make sure the directories exist */
-	g_message ("Checking database directories exist");
+	/* Don't do need_reindex checks for readonly (direct-access) */
+	if ((flags & TRACKER_DB_MANAGER_READONLY) == 0) {
 
-	g_mkdir_with_parents (data_dir, 00755);
-	g_mkdir_with_parents (user_data_dir, 00755);
-	g_mkdir_with_parents (sys_tmp_dir, 00755);
+		/* Make sure the directories exist */
+		g_message ("Checking database directories exist");
 
-	g_message ("Checking database version");
+		g_mkdir_with_parents (data_dir, 00755);
+		g_mkdir_with_parents (user_data_dir, 00755);
+		g_mkdir_with_parents (sys_tmp_dir, 00755);
 
-	version = db_get_version ();
+		g_message ("Checking database version");
 
-	if (version < TRACKER_DB_VERSION_NOW) {
-		g_message ("  A reindex will be forced");
-		need_reindex = TRUE;
-	}
+		version = db_get_version ();
 
-	if (need_reindex) {
-		tracker_db_manager_create_version_file ();
-		tracker_db_manager_set_need_mtime_check (TRUE);
+		if (version < TRACKER_DB_VERSION_NOW) {
+			g_message ("  A reindex will be forced");
+			need_reindex = TRUE;
+		}
+
+		if (need_reindex) {
+			tracker_db_manager_create_version_file ();
+			tracker_db_manager_set_need_mtime_check (TRUE);
+		}
 	}
 
 	g_message ("Checking database files exist");
 
 	for (i = 1; i < G_N_ELEMENTS (dbs); i++) {
 		/* Fill absolute path for the database */
-		dir = location_to_directory (dbs[i].location);
 
+		dir = location_to_directory (dbs[i].location);
 		g_free (dbs[i].abs_filename);
 		dbs[i].abs_filename = g_build_filename (dir, dbs[i].file, NULL);
 
@@ -832,28 +946,50 @@ tracker_db_manager_init (TrackerDBManagerFlags  flags,
 		 * missing, we reindex.
 		 */
 
-		/* No need to check for other files not existing (for
-		 * reindex) if one is already missing.
-		 */
-		if (need_reindex) {
-			continue;
+		if ((flags & TRACKER_DB_MANAGER_READONLY) == 0) {
+			/* No need to check for other files not existing (for
+			 * reindex) if one is already missing.
+			 */
+			if (need_reindex) {
+				continue;
+			}
 		}
 
 		if (!g_file_test (dbs[i].abs_filename, G_FILE_TEST_EXISTS)) {
-			g_message ("Could not find database file:'%s'", dbs[i].abs_filename);
-			g_message ("One or more database files are missing, a reindex will be forced");
-			need_reindex = TRUE;
+			if ((flags & TRACKER_DB_MANAGER_READONLY) == 0) {
+				g_message ("Could not find database file:'%s'", dbs[i].abs_filename);
+				g_message ("One or more database files are missing, a reindex will be forced");
+				need_reindex = TRUE;
+			} else {
+				guint y;
+
+				g_set_error (error,
+				             TRACKER_DB_INTERFACE_ERROR,
+				             TRACKER_DB_OPEN_ERROR,
+				             "Could not find database file:'%s'. One or more database files are missing", dbs[i].abs_filename);
+
+				for (y = 1; y <= i; y++) {
+					g_free (dbs[y].abs_filename);
+					dbs[y].abs_filename = NULL;
+				}
+
+				return FALSE;
+			}
 		}
 	}
 
 	locations_initialized = TRUE;
 
-	/* If we are just initializing to remove the databases,
-	 * return here.
-	 */
-	if ((flags & TRACKER_DB_MANAGER_REMOVE_ALL) != 0) {
-		initialized = TRUE;
-		return TRUE;
+	/* Don't do remove-dbs for readonly (direct-access) */
+	if ((flags & TRACKER_DB_MANAGER_READONLY) == 0) {
+
+		/* If we are just initializing to remove the databases,
+		 * return here.
+		 */
+		if ((flags & TRACKER_DB_MANAGER_REMOVE_ALL) != 0) {
+			initialized = TRUE;
+			return TRUE;
+		}
 	}
 
 	/* Set general database options */
@@ -867,23 +1003,27 @@ tracker_db_manager_init (TrackerDBManagerFlags  flags,
 	 * other things like the nfs lock file.
 	 */
 	if (flags & TRACKER_DB_MANAGER_FORCE_REINDEX || need_reindex) {
+
 		if (flags & TRACKER_DB_MANAGER_READONLY) {
 			/* no reindexing supported in read-only mode (direct access) */
-			return FALSE;
-		}
 
-		if (first_time) {
-			*first_time = TRUE;
-		}
+			g_set_error (error,
+			             TRACKER_DB_INTERFACE_ERROR,
+			             TRACKER_DB_OPEN_ERROR,
+			             "No reindexing supported in read-only mode (direct access)");
 
-		if (!tracker_file_system_has_enough_space (data_dir, TRACKER_DB_MIN_REQUIRED_SPACE, TRUE)) {
 			return FALSE;
 		}
 
 		/* Clear the first-index stamp file */
 		tracker_db_manager_set_first_index_done (FALSE);
 
-		db_recreate_all ();
+		perform_recreate (first_time, &internal_error);
+
+		if (internal_error) {
+			g_propagate_error (error, internal_error);
+			return FALSE;
+		}
 
 		/* Load databases */
 		g_message ("Loading databases files...");
@@ -914,7 +1054,6 @@ tracker_db_manager_init (TrackerDBManagerFlags  flags,
 
 			for (i = 1; i < G_N_ELEMENTS (dbs) && !must_recreate; i++) {
 				struct stat st;
-				GError *error = NULL;
 				TrackerDBStatement *stmt;
 				gchar *busy_status;
 
@@ -930,7 +1069,16 @@ tracker_db_manager_init (TrackerDBManagerFlags  flags,
 					continue;
 				}
 
-				dbs[i].iface = db_interface_create (i);
+				dbs[i].iface = db_interface_create (i, &internal_error);
+
+				if (internal_error) {
+					/* If this already doesn't succeed, then surely the file is
+					 * corrupt. No need to check for integrity anymore. */
+					g_clear_error (&internal_error);
+					must_recreate = TRUE;
+					continue;
+				}
+
 				dbs[i].mtime = tracker_file_get_mtime (dbs[i].abs_filename);
 
 				loaded = TRUE;
@@ -945,17 +1093,19 @@ tracker_db_manager_init (TrackerDBManagerFlags  flags,
 				                                       busy_user_data);
 				g_free (busy_status);
 
-				stmt = tracker_db_interface_create_statement (dbs[i].iface, TRACKER_DB_STATEMENT_CACHE_TYPE_NONE, &error,
+				stmt = tracker_db_interface_create_statement (dbs[i].iface, TRACKER_DB_STATEMENT_CACHE_TYPE_NONE,
+				                                              &internal_error,
 				                                              "PRAGMA integrity_check(1)");
 
-				if (error != NULL) {
-					if (error->domain == TRACKER_DB_INTERFACE_ERROR &&
-					    error->code == TRACKER_DB_QUERY_ERROR) {
+				if (internal_error != NULL) {
+					if (internal_error->domain == TRACKER_DB_INTERFACE_ERROR &&
+					    internal_error->code == TRACKER_DB_QUERY_ERROR) {
 						must_recreate = TRUE;
 					} else {
-						g_critical ("%s", error->message);
+						g_critical ("%s", internal_error->message);
 					}
-					g_error_free (error);
+					g_error_free (internal_error);
+					internal_error = NULL;
 				} else {
 					TrackerDBCursor *cursor = NULL;
 
@@ -976,6 +1126,19 @@ tracker_db_manager_init (TrackerDBManagerFlags  flags,
 					}
 				}
 
+				/* ensure that database has been initialized by an earlier tracker-store start
+				   by checking whether Resource table exists */
+				stmt = tracker_db_interface_create_statement (dbs[i].iface, TRACKER_DB_STATEMENT_CACHE_TYPE_NONE,
+				                                              &internal_error,
+				                                              "SELECT 1 FROM Resource");
+				if (internal_error != NULL) {
+					must_recreate = TRUE;
+					g_error_free (internal_error);
+					internal_error = NULL;
+				} else {
+					g_object_unref (stmt);
+				}
+
 				tracker_db_interface_set_busy_handler (dbs[i].iface, NULL, NULL, NULL);
 			}
 		}
@@ -984,22 +1147,11 @@ tracker_db_manager_init (TrackerDBManagerFlags  flags,
 
 			g_message ("Database severely damaged. We will recreate it and replay the journal if available.");
 
-			if (first_time) {
-				*first_time = TRUE;
-			}
-
-			for (i = 1; i < G_N_ELEMENTS (dbs); i++) {
-				if (dbs[i].iface) {
-					g_object_unref (dbs[i].iface);
-					dbs[i].iface = NULL;
-				}
-			}
-
-			if (!tracker_file_system_has_enough_space (data_dir, TRACKER_DB_MIN_REQUIRED_SPACE, TRUE)) {
+			perform_recreate (first_time, &internal_error);
+			if (internal_error) {
+				g_propagate_error (error, internal_error);
 				return FALSE;
 			}
-
-			db_recreate_all ();
 			loaded = FALSE;
 		}
 
@@ -1026,15 +1178,34 @@ tracker_db_manager_init (TrackerDBManagerFlags  flags,
 	initialized = TRUE;
 
 	if (flags & TRACKER_DB_MANAGER_READONLY) {
-		resources_iface = tracker_db_manager_get_db_interfaces_ro (1,
+		resources_iface = tracker_db_manager_get_db_interfaces_ro (&internal_error, 1,
 		                                                           TRACKER_DB_METADATA);
 		/* libtracker-direct does not use per-thread interfaces */
 		global_iface = resources_iface;
 	} else {
-		resources_iface = tracker_db_manager_get_db_interfaces (1,
+		resources_iface = tracker_db_manager_get_db_interfaces (&internal_error, 1,
 		                                                        TRACKER_DB_METADATA);
 	}
 
+	if (internal_error) {
+		if ((flags & TRACKER_DB_MANAGER_READONLY) == 0) {
+			perform_recreate (first_time, &internal_error);
+			if (!internal_error) {
+				resources_iface = tracker_db_manager_get_db_interfaces (&internal_error, 1,
+				                                                        TRACKER_DB_METADATA);
+			}
+			if (internal_error) {
+				g_propagate_error (error, internal_error);
+				initialized = FALSE;
+				return FALSE;
+			}
+		} else {
+			g_propagate_error (error, internal_error);
+			initialized = FALSE;
+			return FALSE;
+		}
+	}
+
 	tracker_db_interface_set_max_stmt_cache_size (resources_iface,
 	                                              TRACKER_DB_STATEMENT_CACHE_TYPE_SELECT,
 	                                              select_cache_size);
@@ -1088,7 +1259,8 @@ tracker_db_manager_shutdown (void)
 		global_iface = NULL;
 	}
 
-	/* shutdown db interfaces in all threads */
+	/* shutdown db interface in all threads
+	 * this can currently cause critical warnings due to a bug in g_static_private_free */
 	g_static_private_free (&interface_data_key);
 
 	/* Since we don't reference this enum anywhere, we do
@@ -1425,11 +1597,13 @@ tracker_db_manager_get_file (TrackerDB db)
  * returns: (caller-owns): a database connection
  **/
 static TrackerDBInterface *
-tracker_db_manager_get_db_interfaces (gint num, ...)
+tracker_db_manager_get_db_interfaces (GError **error,
+                                      gint num, ...)
 {
-	gint                n_args;
-	va_list                     args;
+	gint n_args;
+	va_list args;
 	TrackerDBInterface *connection = NULL;
+	GError *internal_error = NULL;
 
 	g_return_val_if_fail (initialized != FALSE, NULL);
 
@@ -1438,11 +1612,25 @@ tracker_db_manager_get_db_interfaces (gint num, ...)
 		TrackerDB db = va_arg (args, TrackerDB);
 
 		if (!connection) {
-			connection = tracker_db_interface_sqlite_new (dbs[db].abs_filename);
+			connection = tracker_db_interface_sqlite_new (dbs[db].abs_filename,
+			                                              &internal_error);
+
+			if (internal_error) {
+				g_propagate_error (error, internal_error);
+				connection = NULL;
+				goto end_on_error;
+			}
 
 			db_set_params (connection,
 			               dbs[db].cache_size,
-			               dbs[db].page_size);
+			               dbs[db].page_size,
+			               &internal_error);
+
+			if (internal_error) {
+				g_propagate_error (error, internal_error);
+				connection = NULL;
+				goto end_on_error;
+			}
 
 		} else {
 			db_exec_no_reply (connection,
@@ -1452,17 +1640,22 @@ tracker_db_manager_get_db_interfaces (gint num, ...)
 		}
 
 	}
+
+	end_on_error:
+
 	va_end (args);
 
 	return connection;
 }
 
 static TrackerDBInterface *
-tracker_db_manager_get_db_interfaces_ro (gint num, ...)
+tracker_db_manager_get_db_interfaces_ro (GError **error,
+                                         gint num, ...)
 {
-	gint                n_args;
-	va_list                     args;
+	gint n_args;
+	va_list args;
 	TrackerDBInterface *connection = NULL;
+	GError *internal_error = NULL;
 
 	g_return_val_if_fail (initialized != FALSE, NULL);
 
@@ -1471,29 +1664,45 @@ tracker_db_manager_get_db_interfaces_ro (gint num, ...)
 		TrackerDB db = va_arg (args, TrackerDB);
 
 		if (!connection) {
-			connection = tracker_db_interface_sqlite_new_ro (dbs[db].abs_filename);
+			connection = tracker_db_interface_sqlite_new_ro (dbs[db].abs_filename,
+			                                                 &internal_error);
+
+			if (internal_error) {
+				g_propagate_error (error, internal_error);
+				connection = NULL;
+				goto end_on_error;
+			}
+
 			db_set_params (connection,
 			               dbs[db].cache_size,
-			               dbs[db].page_size);
+			               dbs[db].page_size,
+			               &internal_error);
+
+			if (internal_error) {
+				g_propagate_error (error, internal_error);
+				connection = NULL;
+				goto end_on_error;
+			}
+
 		} else {
 			db_exec_no_reply (connection,
 			                  "ATTACH '%s' as '%s'",
 			                  dbs[db].abs_filename,
 			                  dbs[db].name);
 		}
-
 	}
+
+	end_on_error:
+
 	va_end (args);
 
 	return connection;
 }
 
-
 /**
  * tracker_db_manager_get_db_interface:
- * @db: the database file wanted
  *
- * Request a database connection to the database file @db.
+ * Request a database connection to the database
  *
  * The caller must NOT g_object_unref the result
  *
@@ -1502,6 +1711,7 @@ tracker_db_manager_get_db_interfaces_ro (gint num, ...)
 TrackerDBInterface *
 tracker_db_manager_get_db_interface (void)
 {
+	GError *internal_error = NULL;
 	TrackerDBInterface *interface;
 
 	g_return_val_if_fail (initialized != FALSE, NULL);
@@ -1515,9 +1725,15 @@ tracker_db_manager_get_db_interface (void)
 
 	/* Ensure the interface is there */
 	if (!interface) {
-		interface = tracker_db_manager_get_db_interfaces (1,
+		interface = tracker_db_manager_get_db_interfaces (&internal_error, 1,
 		                                                  TRACKER_DB_METADATA);
 
+		if (internal_error) {
+			g_critical ("Error opening database: %s", internal_error->message);
+			g_error_free (internal_error);
+			return NULL;
+		}
+
 		tracker_db_interface_sqlite_fts_init (interface, FALSE);
 
 
