@@ -148,6 +148,8 @@ static TrackerExtractData data[] = {
 	{ "video/3gpp", extract_gstreamer_guess },
 	{ "video/mp4", extract_gstreamer_guess },
 	{ "video/x-ms-asf", extract_gstreamer_guess },
+	/* Added to fix broken Real media detection, see MeeGo bug #18159 */
+	{ "application/vnd.rn-realmedia", extract_gstreamer_video },
 	{ NULL, NULL }
 };
 
