@@ -1220,14 +1220,12 @@ get_id3v24_tags (id3v24frame           frame,
 	case ID3V24_COMM: {
 		gchar *word;
 		gchar text_encode;
-		const gchar *text_language;
 		const gchar *text_desc;
 		const gchar *text;
 		guint offset;
 		gint text_desc_len;
 
 		text_encode   =  data[pos + 0]; /* $xx */
-		text_language = &data[pos + 1]; /* $xx xx xx */
 		text_desc     = &data[pos + 4]; /* <text string according to encoding> $00 (00) */
 		text_desc_len = id3v2_strlen (text_encode, text_desc, csize - 4);
 
@@ -1408,14 +1406,12 @@ get_id3v23_tags (id3v24frame           frame,
 	case ID3V24_COMM: {
 		gchar *word;
 		gchar text_encode;
-		const gchar *text_language;
 		const gchar *text_desc;
 		const gchar *text;
 		guint offset;
 		gint text_desc_len;
 
 		text_encode   =  data[pos + 0]; /* $xx */
-		text_language = &data[pos + 1]; /* $xx xx xx */
 		text_desc     = &data[pos + 4]; /* <text string according to encoding> $00 (00) */
 		text_desc_len = id3v2_strlen (text_encode, text_desc, csize - 4);
 
@@ -1687,11 +1683,9 @@ parse_id3v24 (const gchar           *data,
 	gint unsync;
 	gint ext_header;
 	gint experimental;
-	gint footer;
 	guint tsize;
 	guint pos;
 	guint ext_header_size;
-	guint padding;
 
 	if ((size < 16) ||
 	    (data[0] != 0x49) ||
@@ -1705,7 +1699,6 @@ parse_id3v24 (const gchar           *data,
 	unsync = (data[5] & 0x80) > 0;
 	ext_header = (data[5] & 0x40) > 0;
 	experimental = (data[5] & 0x20) > 0;
-	footer = (data[5] & 0x10) > 0;
 	tsize = (((data[6] & 0x7F) << 21) |
 	         ((data[7] & 0x7F) << 14) |
 	         ((data[8] & 0x7F) << 7) |
@@ -1716,7 +1709,6 @@ parse_id3v24 (const gchar           *data,
 	}
 
 	pos = 10;
-	padding = 0;
 
 	if (ext_header) {
 		ext_header_size = (((data[10] & 0x7F) << 21) |
